package com.redhat.thermostat.tools.eclipse.plugin.tests.model;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.io.ByteArrayOutputStream;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.List;

import org.junit.Test;

import com.redhat.thermostat.tools.eclipse.plugin.model.Bundle;
import com.redhat.thermostat.tools.eclipse.plugin.model.Command;
import com.redhat.thermostat.tools.eclipse.plugin.model.Environments;
import com.redhat.thermostat.tools.eclipse.plugin.model.Extension;
import com.redhat.thermostat.tools.eclipse.plugin.model.Option;
import com.redhat.thermostat.tools.eclipse.plugin.model.Options;
import com.redhat.thermostat.tools.eclipse.plugin.model.Plugin;
import com.redhat.thermostat.tools.eclipse.plugin.model.PluginModelReaderWriter;

public class PluginModelReaderWriterTest {

    @Test
    public void testReadingCommandFromXml() {
        String document = "<?xml version='1.0'?>\n"
                + "<plugin xmlns=\"http://icedtea.classpath.org/thermostat/plugins/v1.0\"\n"
                + "    xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n"
                + "    xsi:schemaLocation=\"http://icedtea.classpath.org/thermostat/plugins/v1.0 thermostat-plugin.xsd\">\n"
                + "  <commands>\n"
                + "    <command>\n"
                + "      <name>foo</name>\n"
                + "      <usage>foo bar</usage>\n"
                + "      <description>foos the bar if possible</description>\n"
                + "      <arguments>\n"
                + "        <argument>bar</argument>\n"
                + "      </arguments>\n"
                + "      <options>\n"
                + "        <option>\n"
                + "          <short>a</short>\n"
                + "        </option>\n"
                + "      </options>\n"
                + "      <environments>\n"
                + "        <environment>cli</environment>\n"
                + "      </environments>\n"
                + "      <bundles>\n"
                + "        <bundle><symbolic-name>foo</symbolic-name><version>1</version></bundle>\n"
                + "      </bundles>\n"
                + "    </command>\n"
                + "  </commands>\n"
                + "</plugin>\n";

        PluginModelReaderWriter pluginModel = new PluginModelReaderWriter(null, "<stdin>");
        Plugin model = pluginModel.loadModel(document);

        Command foo = model.getCommands().get(0);

        assertNotNull(foo);

        assertEquals("foo", foo.getName());
        assertEquals("foo bar", foo.getUsage());
        assertEquals("foos the bar if possible", foo.getDescription());

        Options options = foo.getOptions();
        assertNotNull(options);

        List<Option> simpleOptions = options.getOptions();
        assertNotNull(simpleOptions);
        assertEquals(1, simpleOptions.size());

        assertTrue(foo.getEnvironments().isCli());
        assertFalse(foo.getEnvironments().isShell());

        List<Bundle> bundles = foo.getBundles();
        assertEquals(1, bundles.size());

        Bundle bundle = bundles.get(0);
        assertEquals("foo", bundle.getSymbolicName());
        assertEquals("1", bundle.getVersion());
    }

    @Test
    public void testReadingExtensionFromXml() {
        String document = "<?xml version='1.0'?>\n"
                + "<plugin xmlns=\"http://icedtea.classpath.org/thermostat/plugins/v1.0\"\n"
                + "    xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n"
                + "    xsi:schemaLocation=\"http://icedtea.classpath.org/thermostat/plugins/v1.0 thermostat-plugin.xsd\">\n"
                + "  <extensions>\n"
                + "    <extension>\n"
                + "      <name>foo</name>\n"
                + "      <bundles>\n"
                + "        <bundle><symbolic-name>foo</symbolic-name><version>1</version></bundle>\n"
                + "      </bundles>\n"
                + "    </extension>\n"
                + "  </extensions>\n"
                + "</plugin>\n";

        PluginModelReaderWriter model = new PluginModelReaderWriter(null, "stdin");
        Plugin plugin = model.loadModel(document);

        List<Extension> extensions = plugin.getExtensions();
        assertEquals(1, extensions.size());
    }

    @Test
    public void testWritingCommandToXml() {
        String emptyDocument = "<?xml version='1.0'?>\n"
                + "<plugin xmlns=\"http://icedtea.classpath.org/thermostat/plugins/v1.0\"\n"
                + "    xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n"
                + "    xsi:schemaLocation=\"http://icedtea.classpath.org/thermostat/plugins/v1.0 thermostat-plugin.xsd\">\n"
                + "</plugin>";

        String expectedDocument = "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>\n"
                + "<plugin xmlns=\"http://icedtea.classpath.org/thermostat/plugins/v1.0\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:schemaLocation=\"http://icedtea.classpath.org/thermostat/plugins/v1.0 http://icedtea.classpath.org/thermostat/docs/1.0/thermostat-plugin.xsd\">\n"
                + "    <commands>\n"
                + "        <command>\n"
                + "            <name>foo</name>\n"
                + "            <description>foos the bar if possible</description>\n"
                + "            <usage>foo bar</usage>\n"
                + "            <arguments>\n"
                + "                <argument>bar</argument>\n"
                + "            </arguments>\n"
                + "            <options>\n"
                + "                <option>\n"
                + "                    <short>a</short>\n"
                + "                    <required>true</required>\n"
                + "                </option>\n"
                + "            </options>\n"
                + "            <environments>\n"
                + "                <environment>cli</environment>\n"
                + "            </environments>\n"
                + "            <bundles>\n"
                + "                <bundle>\n"
                + "                    <symbolic-name>foo</symbolic-name>\n"
                + "                    <version>1</version>\n"
                + "                </bundle>\n"
                + "            </bundles>\n"
                + "        </command>\n"
                + "    </commands>\n"
                + "</plugin>\n";

        PluginModelReaderWriter pluginModel = new PluginModelReaderWriter(null, "<stdin>");
        Plugin model = pluginModel.loadModel(emptyDocument);

        // just to verify that the initial model is empty before adding the actual command
        assertEquals(0, model.getCommands().size());

        Command command = new Command();
        command.setName("foo");
        command.setUsage("foo bar");
        command.setDescription("foos the bar if possible");

        String argument = "bar";
        command.addArgument(argument);

        Options options = new Options();
        command.setOptions(options);

        List<Option> simpleOptions = new ArrayList<>();
        options.setOptions(simpleOptions);

        Option optionA = new Option();
        optionA.setShortName("a");
        optionA.setRequired(true);
        simpleOptions.add(optionA);

        Environments environments = new Environments();
        environments.setCli(true);
        environments.setShell(false);
        command.setEnvironments(environments);

        Bundle bundle = new Bundle();
        bundle.setSymbolicName("foo");
        bundle.setVersion("1");
        command.addBundle(bundle);

        model.addCommand(command);

        ByteArrayOutputStream bytes = new ByteArrayOutputStream();
        pluginModel.saveModel(model, bytes);
        String generatedDocument = new String(bytes.toByteArray(), StandardCharsets.UTF_8);

        assertEquals(expectedDocument, generatedDocument);
    }
}
